@@ -1,26 +1,26 @@
-# std vs
-from vapoursynth import core
-import vapoursynth as vs
-import os
 import functools
-# pip packages
-from pyd2v import D2V
-# pvsfunc
-from pvsfunc.helpers import anti_file_prefix, get_mime_type, get_video_codec, get_d2v, calculate_par, calculate_aspect_ratio
+import itertools
+import os
 
+import vapoursynth as vs
+from pyd2v import D2V
+from vapoursynth import core
+
+from pvsfunc.helpers import anti_file_prefix, get_mime_type, get_video_codec, get_d2v, fps_reset, \
+    calculate_par, calculate_aspect_ratio
 
 CODEC_SOURCER_MAP = {
     "IMAGE": "core.imwri.Read",
     "V_MPEG1": "core.d2v.Source",
     "V_MPEG2": "core.d2v.Source",
-    # codecs not listed here will default to `core.lsmas.LWLibavSource`
 }
 
 SOURCER_ARGS_MAP = {
     "core.imwri.Read": {
         # disable the alpha channel as it often causes an incompatibility
         # with some functions due to the extra channel
-        "alpha": False
+        "alpha": False,
+        "firstnum": 0
     },
     "core.d2v.Source": {
         # ignore rff (pulldown flags) as we do not want it to interlace
@@ -60,17 +60,41 @@ class PSourcer:
         self.debug = debug
         self.clip = None
         self.file_path = anti_file_prefix(file_path)
-        self.file_type = get_mime_type(self.file_path)
+        self.mime_type = get_mime_type(self.file_path)
         self.video_codec = get_video_codec(self.file_path)
-        if self.file_type.startswith("image/"):
+        if self.video_codec == -1:
+            raise ValueError("pvsfunc.PSourcer: File path supplied does not exist")
+        elif self.video_codec == -2:
+            raise ValueError("pvsfunc.PSourcer: File supplied does not have a Video or Image track")
+        if self.mime_type.startswith("image/"):
+            # we do this after get_video_codec so it checks if an image
+            # track actually exists, and not just an empty image container
             self.video_codec = "IMAGE"
-        self.sourcer = self.get_sourcer(self.video_codec)
+        self.sourcer = CODEC_SOURCER_MAP.get(self.video_codec, "core.lsmas.LWLibavSource")
         # sourcer preparations
         if self.sourcer == "core.d2v.Source":
             # make sure a d2v file for this video exists
             self.file_path = get_d2v(self.file_path)
+        elif self.sourcer == "core.lsmas.LWLibavSource":
+            # destroy the container-set fps
+            self.file_path = fps_reset(self.file_path)
         # load video to clip using sourcer
-        self.clip = eval(self.sourcer)(self.file_path, **SOURCER_ARGS_MAP[self.sourcer])
+        while True:
+            try:
+                func = core
+                for split in self.sourcer.replace("core.", "").split("."):
+                    func = getattr(func, split)
+                self.clip = func(self.file_path, **SOURCER_ARGS_MAP[self.sourcer])
+                break
+            except vs.Error as e:
+                if self.sourcer == "core.imwri.Read" and "Read: No files matching the given pattern exist" in str(e):
+                    # default uses first num of 0, maybe the user's first image is 1, let's not annoy them
+                    # and just dynamically handle that
+                    if SOURCER_ARGS_MAP[self.sourcer]["firstnum"] == 1:
+                        raise
+                    SOURCER_ARGS_MAP[self.sourcer]["firstnum"] = 1
+                else:
+                    raise
         # post sourcing preparations
         if self.sourcer == "core.d2v.Source":
             # ========================================================================= #
@@ -84,25 +108,27 @@ class PSourcer:
             d2v = D2V(self.file_path)
             # get every frames' flag data, this contains information on displaying frames
             # add vob and cell number to each frames flag data as well
-            flags = [f for l in [
-                [dict(**y, vob=x["vob"], cell=x["cell"]) for y in x["flags"]] for x in d2v.data
-            ] for f in l]
+            flags = [[dict(**y, vob=d["vob"], cell=d["cell"]) for y in d["flags"]] for d in d2v.data]
+            flags = list(itertools.chain.from_iterable(flags))  # flatten list of lists
             # Get pulldown cycle
             # todo ; get an mpeg2 that uses Pulldown metadata (rff flags) that ISN'T Pulldown 2:3 to test math
             #        this math seems pretty far fetched, if we can somehow obtain the Pulldown x:x:...
             #        string that mediainfo can get, then calculating it can be much easier and more efficient.
-            pulldown_cycle = [n for n,f in enumerate(flags) if f["tff"] and f["rff"]]
-            if not pulldown_cycle:
-                pulldown_cycle = 0  # a 0 would be better than an empty list
+            pulldown_cycle = [n for n, f in enumerate(flags) if f["tff"] and f["rff"]]
+            if not pulldown_cycle or pulldown_cycle == [0]:
+                pulldown_cycle = 0  # a 0 would be better than an empty list/list with only a 0
             else:
                 # pair every 2 frame indexes together
                 pulldown_cycle = list(zip(pulldown_cycle[::2], pulldown_cycle[1::2]))
                 # subtract the right index with the left index to calculate the cycle
-                pulldown_cycle = [r - l for l,r in pulldown_cycle]
+                pulldown_cycle = [right - left for left, right in pulldown_cycle]
                 # get the most common cycle (+1), interlaced sections in variable scan-type content messes up the
                 # results, so the only way around it is to get the most common entry.
                 pulldown_cycle = max(set(pulldown_cycle), key=pulldown_cycle.count) + 1
             # set various data used for debugging (if debug enabled)
+            coded_pictures = None
+            progressive_percent = None
+            pulldown_count = None
             if self.debug:
                 coded_pictures = len(flags)
                 progressive_percent = (sum(1 for f in flags if f["progressive_frame"]) / len(flags)) * 100
@@ -114,12 +140,12 @@ class PSourcer:
                 # - mix of progressive and interlaced sections
                 # 1. fix the frame rate of the progressive sections by applying it's pulldown
                 #    we fix it by duplicating the pulldown specified frames rather than as fields
-                pulldown_indexes = [n for n,f in enumerate(flags) if f["progressive_frame"] and f["rff"] and f["tff"]]
+                pulldown_indexes = [n for n, f in enumerate(flags) if f["progressive_frame"] and f["rff"] and f["tff"]]
                 if pulldown_indexes:
                     self.clip = core.std.DuplicateFrames(clip=self.clip, frames=pulldown_indexes)
                 # 2. apply the changes above to the flag list to match the fixed clip
                 flags = [f for sl in [(
-                    [f,dict(**{**f, **{"progressive_frame": True, "rff": False, "tff": False}})]
+                    [f, dict(**{**f, **{"progressive_frame": True, "rff": False, "tff": False}})]
                     if f["progressive_frame"] and f["rff"] and f["tff"] else [f]
                 ) for f in flags] for f in sl]
             else:
@@ -134,6 +160,7 @@ class PSourcer:
                         fpsnum=self.clip.fps.numerator - (self.clip.fps.numerator / pulldown_cycle),
                         fpsden=self.clip.fps.denominator
                     )
+
             # ========================================================================= #
             #  Store flags in each frame's props                                        #
             # ------------------------------------------------------------------------- #
@@ -142,51 +169,40 @@ class PSourcer:
             # list of the props available at any time with core.text.FrameProps().      #
             # All props will be either an int, int-style bool, or string.               #
             # ========================================================================= #
-            def set_flag_props(n, f, c, fl):
-                for k, v in fl[n].items():
-                    if isinstance(v, bool):
-                        v = 1 if v else 0
-                    if isinstance(v, bytes):
-                        v = v.decode("utf-8")
-                    c = core.std.SetFrameProp(c, **{
-                        ("intval" if isinstance(v, int) else "data"): v
-                    }, prop=f"PVSFlag{k.title().replace('_', '')}")
-                return c[n]
             self.clip = core.std.FrameEval(
                 self.clip,
                 functools.partial(
-                    set_flag_props,
+                    self._set_flag_props,
                     c=self.clip,
                     fl=flags
                 ),
                 prop_src=self.clip
             )
             vob_indexes = [index for _, index in {f["vob"]: n for n, f in enumerate(flags)}.items()]
-            vob_indexes = [f"{(0 if n == 0 else (vob_indexes[n-1] + 1))}-{i}" for n,i in enumerate(vob_indexes)]
+            vob_indexes = [f"{(0 if n == 0 else (vob_indexes[n - 1] + 1))}-{i}" for n, i in enumerate(vob_indexes)]
             self.clip = core.std.SetFrameProp(self.clip, prop="PVSVobIdIndexes", data=" ".join(vob_indexes))
             if self.debug:
                 # fps
                 fps = self.clip.fps
-                if self.clip.fps.numerator == 25:
+                if self.clip.fps.numerator == 25 and self.clip.fps.denominator == 1:
                     fps = "PAL"
-                elif self.clip.fps.numerator == 30000:
+                elif self.clip.fps.numerator == 30000 and self.clip.fps.denominator == 1001:
                     fps = "NTSC"
-                elif self.clip.fps.numerator == 24:
+                elif self.clip.fps.numerator == 24 and self.clip.fps.denominator == 1:
                     fps = "FILM"
                 # aspect ratio
-                ar = d2v.settings['Aspect_Ratio']
-                ar_n = [int(x) for x in ar.split(':')]
-                par = calculate_par(self.clip.width, self.clip.height, *ar_n)
+                dar = d2v.settings["Aspect_Ratio"]
+                par = calculate_par(self.clip.width, self.clip.height, *[int(x) for x in dar.split(":")])
                 sar = calculate_aspect_ratio(self.clip.width, self.clip.height)
                 self.clip = core.text.Text(
                     self.clip,
                     " " + (" \n ".join([
                         f"{os.path.basename(self.file_path)}",
-                        f"{fps}, Loaded with {str(self.sourcer)}",
+                        f"Loaded with {str(self.sourcer)}",
                         f"- {coded_pictures:,} coded pictures, which {progressive_percent:.2f}% of are Progressive",
                         f"- {pulldown_count:,} frames are asking for pulldown{f' which occurs every {pulldown_cycle} frames' if pulldown_cycle else ''}",
                         f"- {coded_pictures + pulldown_count:,} total frames after pulldown flags are honored",
-                        f"DAR: {ar}  SAR: {sar}  PAR: {par}"
+                        f"DAR: {dar}  SAR: {sar}  PAR: {par}  FPS: {fps}"
                     ])) + " ",
                     alignment=7
                 )
@@ -201,7 +217,13 @@ class PSourcer:
             self.clip = core.std.SetFrameProp(self.clip, prop=f"PVS{k}", data=v)
 
     @staticmethod
-    def get_sourcer(video_codec):
-        """Get clip sourcer function based on video codec"""
-        # default to FFMPEG-based sourcer for wide compatibility
-        return CODEC_SOURCER_MAP.get(video_codec, "core.lsmas.LWLibavSource")
+    def _set_flag_props(n, f, c, fl):
+        for key, value in fl[n].items():
+            if isinstance(value, bool):
+                value = 1 if value else 0
+            if isinstance(value, bytes):
+                value = value.decode("utf-8")
+            c = core.std.SetFrameProp(c, **{
+                ("intval" if isinstance(value, int) else "data"): value
+            }, prop=f"PVSFlag{key.title().replace('_', '')}")
+        return c[n]
